@@ -16,8 +16,6 @@
   Authors: Wu Jiaxu (wujiaxu@sogou-inc.com)
 */
 
-#include <openssl/ssl.h>
-#include <openssl/sha.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
@@ -30,6 +28,7 @@
 #include <vector>
 #include <string>
 #include <algorithm>
+#include <openssl/ssl.h>
 #include "list.h"
 #include "rbtree.h"
 #include "WFGlobal.h"
@@ -105,12 +104,12 @@ struct RouteParams
 {
 	TransportType transport_type;
 	const struct addrinfo *addrinfo;
-	uint64_t sha1_64;
+	uint64_t key;
 	SSL_CTX *ssl_ctx;
+	unsigned int max_connections;
 	int connect_timeout;
-	int ssl_connect_timeout;
 	int response_timeout;
-	size_t max_connections;
+	int ssl_connect_timeout;
 	bool use_tls_sni;
 	const std::string& hostname;
 };
@@ -124,7 +123,7 @@ public:
 	std::mutex mutex;
 	std::vector<CommSchedTarget *> targets;
 	struct list_head breaker_list;
-	uint64_t sha1_64;
+	uint64_t key;
 	int nleft;
 	int nbreak;
 
@@ -214,7 +213,7 @@ int RouteResultEntry::init(const struct RouteParams *params)
 		{
 			this->targets.push_back(target);
 			this->request_object = target;
-			this->sha1_64 = params->sha1_64;
+			this->key = params->key;
 			return 0;
 		}
 
@@ -227,7 +226,7 @@ int RouteResultEntry::init(const struct RouteParams *params)
 		if (this->add_group_targets(params) >= 0)
 		{
 			this->request_object = this->group;
-			this->sha1_64 = params->sha1_64;
+			this->key = params->key;
 			return 0;
 		}
 
@@ -384,6 +383,20 @@ static inline bool __addr_less(const struct addrinfo *x, const struct addrinfo *
 	return __addr_cmp(x, y) < 0;
 }
 
+static uint64_t __fnv_hash(const unsigned char *data, size_t size)
+{
+	uint64_t hash = 14695981039346656037ULL;
+
+	while (size)
+	{
+		hash ^= (const uint64_t)*data++;
+		hash *= 1099511628211ULL;
+		size--;
+	}
+
+	return hash;
+}
+
 static uint64_t __generate_key(TransportType type,
 							   const struct addrinfo *addrinfo,
 							   const std::string& other_info,
@@ -391,13 +404,12 @@ static uint64_t __generate_key(TransportType type,
 							   const std::string& hostname)
 {
 	std::string buf((const char *)&type, sizeof (TransportType));
-	uint64_t sha1[3];
-	SHA_CTX ctx;
+	unsigned int max_conn = ep_params->max_connections;
 
 	if (!other_info.empty())
 		buf += other_info;
 
-	buf.append((const char *)&ep_params->max_connections, sizeof (size_t));
+	buf.append((const char *)&max_conn, sizeof (unsigned int));
 	buf.append((const char *)&ep_params->connect_timeout, sizeof (int));
 	buf.append((const char *)&ep_params->response_timeout, sizeof (int));
 	if (type == TT_TCP_SSL)
@@ -435,10 +447,7 @@ static uint64_t __generate_key(TransportType type,
 		buf.append((const char *)addrinfo->ai_addr, addrinfo->ai_addrlen);
 	}
 
-	SHA1_Init(&ctx);
-	SHA1_Update(&ctx, buf.c_str(), buf.size());
-	SHA1_Final((unsigned char *)sha1, &ctx);
-	return sha1[1];
+	return __fnv_hash((const unsigned char *)buf.c_str(), buf.size());
 }
 
 RouteManager::~RouteManager()
@@ -457,12 +466,12 @@ RouteManager::~RouteManager()
 int RouteManager::get(TransportType type,
 					  const struct addrinfo *addrinfo,
 					  const std::string& other_info,
-					  const struct EndpointParams *endpoint_params,
+					  const struct EndpointParams *ep_params,
 					  const std::string& hostname,
 					  RouteResult& result)
 {
-	uint64_t sha1_64 = __generate_key(type, addrinfo, other_info,
-									  endpoint_params, hostname);
+	uint64_t key = __generate_key(type, addrinfo, other_info,
+								  ep_params, hostname);
 	struct rb_node **p = &cache_.rb_node;
 	struct rb_node *parent = NULL;
 	RouteResultEntry *bound = NULL;
@@ -473,7 +482,7 @@ int RouteManager::get(TransportType type,
 	{
 		parent = *p;
 		entry = rb_entry(*p, RouteResultEntry, rb);
-		if (sha1_64 <= entry->sha1_64)
+		if (key <= entry->key)
 		{
 			bound = entry;
 			p = &(*p)->rb_left;
@@ -482,7 +491,7 @@ int RouteManager::get(TransportType type,
 			p = &(*p)->rb_right;
 	}
 
-	if (bound && bound->sha1_64 == sha1_64)
+	if (bound && bound->key == key)
 	{
 		entry = bound;
 		entry->check_breaker();
@@ -497,19 +506,19 @@ int RouteManager::get(TransportType type,
 			static SSL_CTX *client_ssl_ctx = WFGlobal::get_ssl_client_ctx();
 
 			ssl_ctx = client_ssl_ctx;
-			ssl_connect_timeout = endpoint_params->ssl_connect_timeout;
+			ssl_connect_timeout = ep_params->ssl_connect_timeout;
 		}
 
 		struct RouteParams params = {
 			.transport_type			=	type,
 			.addrinfo 				= 	addrinfo,
-			.sha1_64				=	sha1_64,
+			.key					=	key,
 			.ssl_ctx 				=	ssl_ctx,
-			.connect_timeout		=	endpoint_params->connect_timeout,
+			.max_connections		=	(unsigned int)ep_params->max_connections,
+			.connect_timeout		=	ep_params->connect_timeout,
+			.response_timeout		=	ep_params->response_timeout,
 			.ssl_connect_timeout	=	ssl_connect_timeout,
-			.response_timeout		=	endpoint_params->response_timeout,
-			.max_connections		=	endpoint_params->max_connections,
-			.use_tls_sni			=	endpoint_params->use_tls_sni,
+			.use_tls_sni			=	ep_params->use_tls_sni,
 			.hostname				=	hostname,
 		};
 
